// Copyright (c) 2005  INRIA Sophia-Antipolis (France).
// All rights reserved.
//
// This file is part of cgal-python; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; version 2.1 of the License.
// See the file LICENSE.LGPL distributed with cgal-python.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// $Id: Py_Min_circle_2_doc.h 125 2006-06-06 16:27:38Z nmeskini $
// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal-python/trunk/cgal-python/bindings/Geometric_Optimisation/Py_Min_circle_2_doc.h $
//
// Author(s)     : Naceur Meskini
//========================================================================
#ifndef MIN_CIRCLE_DOCS_H
#define MIN_CIRCLE_DOCS_H
//=======================================================
// doc of Min_circle_2 class
//=======================================================

const char* Min_circle_2_doc ="\
An object of the class Min_circle_2 is the unique circle of smallest \n\
area enclosing a finite (multi)set of points in two-dimensional euclidean space ExE.\n\
For a point set P we denote by mc(P) the smallest circle that contains all points of P. \n\
Note that mc(P) can be degenerate, i.e. mc(P)=  if P= , mc(P)={p} if P={p}, \n\
and mc(P) = { (1-l)p + l q | 0 <= l <= 1 } if P={p,q}.\n\
An inclusion-minimal subset S of P with mc(S)=mc(P) is called a support set, \n\
the points in S are the support points. A support set has size at most five, \n\
and all its points lie on the boundary of mc(P). In general, \n\
neither the support set nor its size are necessarily unique. \n\
For more details see a C++ doc:\n\
http://www.cgal.org/Manual/3.2/doc_html/cgal_manual/Optimisation_ref/Class_Min_circle_2.html";

const char* Min_circle_init_doc ="\
	min_circle = Min_circle_2() \n\
	initializes min_circle to mc(), the empty set.\n\
	Postcondition: min_circle.is_empty() = true.\n\n\
	min_circle = Min_circle_2(Point_2 p) \n\
	initializes min_circle to mc({p}), the set {p}.\n\
	Postcondition: min_circle.is_degenerate() = true.\n\n\
	min_circle = Min_circle_2(Point_2 p1,Point_2 p2)\n\
	initializes min_circle to mc({p1,p2}), the circle with diameter equal to the segment connecting p1 and p2.\n\n\
	min_circle = Min_circle_2(Point_2 p1,Point_2 p2,Point_2 p3)\n\
	initializes min_circle to mc({p1,p2,p3}).\n\n\
	min_circle = Min_circle_2(list l)\n\
	initializes min_circle with a list of points l.";

const char* Min_circle_support_point_doc ="\
	min_circle.support_point ( int i) -> Point_2\n\
	returns the i-th support point of min_circle. \n\
	Between two modifying operations (see below) any call to min_circle.\n\
	support_point(i) with the same i returns the same point.\n\
	Precondition: 0 <= i < min_circle.number_of_support_points().";

const char* Min_circle_number_of_points_doc ="\
	min_circle.number_of_points( self) -> int\n\
	returns the number of points of min_circle, i.e. |P|.";


const char* Min_circle_number_of_support_points_doc ="\
	min_circle.number_of_support_points( self ,) -> int\n\
	returns the number of support points of min_circle, i.e. |S|.";

const char* Min_circle_circle_doc ="\
	min_circle.circle( self) -> Circle\n\
	returns the current circle of min_circle.";

const char* Min_circle_insert_doc ="\
	min_circle.insert( self , Point_2 p) -> void\n\
	inserts p into min_circle and recomputes the smallest enclosing circle.\n\n\
	min_circle.insert( self , list l) -> void\n\
	inserts a list of points l into min_circle.";


const char* Min_circle_bounded_side_doc ="\
	min_circle.bounded_side( self , Point_2 p) -> Bounded_side\n\
	returns CGAL.Kernel.ON_BOUNDED_SIDE, CGAL.Kernel.ON_BOUNDARY, \n\
	or CGAL.Kernel.ON_UNBOUNDED_SIDE iff p lies properly inside, on \n\
	the boundary, or properly outside of min_circle, resp.";

const char* Min_circle_has_on_bounded_side_doc ="\
	min_circle.has_on_bounded_side( self , Point_2 p) -> bool\n\
	returns true, iff p lies properly inside min_circle.";

const char* Min_circle_has_on_unbounded_side_doc ="\
	min_circle.has_on_unbounded_side( self , Point_2 p) -> bool\n\
	returns true, iff p lies properly outside of min_circle.";

const char* Min_circle_has_on_boundary_doc ="\
	min_circle.has_on_boundary( self , Point_2 p) -> bool\n\
	returns true, iff p lies on the boundary of min_circle.";

const char* Min_circle_is_empty_doc ="\
	min_circle.is_empty( self ) -> bool\n\
	returns true, iff min_circle is empty( self ,this implies degeneracy).";

const char* Min_circle_is_degenerate_doc ="\
	min_circle.is_degenerate( self ) -> bool\n\
	returns true, iff min_circle is degenerate, i.e. \n\
	if min_circle is empty equivalently if the number of support points is less than 2.";

const char* Min_circle_clear_doc ="\
	deletes all points in min_circle and sets min_circle to the empty set.\n\
	Postcondition: min_circle.is_empty() = true.";

const char* Min_circle_is_valid_doc ="\
	min_circle.is_valid( self , bool verbose = false, int level = 0) -> bool\n\
	returns true, iff min_circle contains all points of its defining set P. \n\
	If verbose is true, some messages concerning the performed checks \n\
	are written to standard error stream. The second parameter level is\n\
	not used, we provide it only for consistency with interfaces of other classes.";

#endif // MIN_CIRCLE_DOCS_H


