// Copyright (c) 2005  INRIA Sophia-Antipolis (France).
// All rights reserved.
//
// This file is part of cgal-python; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; version 2.1 of the License.
// See the file LICENSE.LGPL distributed with cgal-python.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// $Id: Py_Min_ellipse_2_doc.h 125 2006-06-06 16:27:38Z nmeskini $
// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal-python/trunk/cgal-python/bindings/Geometric_Optimisation/Py_Min_ellipse_2_doc.h $
//
// Author(s)     : Naceur Meskini
//========================================================================
#ifndef MIN_ELLIPSE_DOCS_H
#define MIN_ELLIPSE_DOCS_H
//=======================================================
// doc of Min_ellipse_2 class
//=======================================================

const char* Min_ellipse_2_doc ="\
An object of the class Min_ellipse_2 is the unique ellipse of smallest \n\
area enclosing a finite (multi)set of points in two-dimensional euclidean space   2.\n\
For a point set P we denote by me(P) the smallest ellipse that contains all points of P. \n\
Note that me(P) can be degenerate, i.e. me(P)=  if P= , me(P)={p} if P={p}, \n\
and me(P) = { (1-l)p + l q | 0 <= l <= 1 } if P={p,q}.\n\
An inclusion-minimal subset S of P with me(S)=me(P) is called a support set, \n\
the points in S are the support points. A support set has size at most five, \n\
and all its points lie on the boundary of me(P). In general, \n\
neither the support set nor its size are necessarily unique. \n\
For more details see a C++ doc:\n\
http://www.cgal.org/Manual/3.2/doc_html/cgal_manual/Optimisation_ref/Class_Min_ellipse_2.html";

const char* Min_ellipse_init_doc ="\
	min_ellipse = Min_ellipse_2(Point_2 p) \n\
	initializes min_ellipse to me({p}), the set {p}.\n\
	Postcondition: min_ellipse.is_degenerate() = true.\n\n\
	min_ellipse = Min_ellipse_2(Point_2 p1,Point_2 p2)\n\
	initializes min_ellipse to me({p,q}), the set { (1-l)p + l q | 0 <= l <= 1 }.\n\
	Postcondition: min_ellipse.is_degenerate() = true.\n\n\
	min_ellipse = Min_ellipse_2(Point_2 p1,Point_2 p2,Point_2 p3)\n\
	initializes min_ellipse to me({p1,p2,p3}).\n\n\
	min_ellipse = Min_ellipse_2(Point_2 p1,Point_2 p2,Point_2 p3,Point_2 p4)\n\
	initializes min_ellipse to me({p1,p2,p3,p4}).\n\n\
	min_ellipse = Min_ellipse_2(Point_2 p1,Point_2 p2,Point_2 p3,Point_2 p4,Point_2 p5)\n\
	initializes min_ellipse to me({p1,p2,p3,p4,p5}).";

const char* Min_ellipse_support_point_doc ="\
	min_ellipse.support_point ( int i) -> Point_2\n\
	returns the i-th support point of min_ellipse. \n\
	Between two modifying operations (see below) any call to min_ellipse.\n\
	support_point(i) with the same i returns the same point.\n\
	Precondition: 0 i< min_ellipse.number_of_support_points().";

const char* Min_ellipse_number_of_points_doc ="\
	min_ellipse.number_of_points( self) -> int\n\
	returns the number of points of min_ellipse, i.e. |P|.";


const char* Min_ellipse_number_of_support_points_doc ="\
	min_ellipse.number_of_support_points( self ,) -> int\n\
	returns the number of support points of min_ellipse, i.e. |S|.";

const char* Min_ellipse_ellipse_doc ="\
	min_ellipse.ellipse( self) -> Ellipse\n\
	returns the current ellipse of min_ellipse.";

const char* Min_ellipse_insert_doc ="\
	min_ellipse.insert( self , Point_2 p) -> void\n\
	inserts p into min_ellipse and recomputes the smallest enclosing ellipse.\n\n\
	min_ellipse.insert( self , list l) -> void\n\
	inserts a list of points l into min_ellipse.";


const char* Min_ellipse_bounded_side_doc ="\
	min_ellipse.bounded_side( self , Point_2 p) -> Bounded_side\n\
	returns CGAL.Kernel.ON_BOUNDED_SIDE, CGAL.Kernel.ON_BOUNDARY, \n\
	or CGAL.Kernel.ON_UNBOUNDED_SIDE iff p lies properly inside, on \n\
	the boundary, or properly outside of min_ellipse, resp.";

const char* Min_ellipse_has_on_bounded_side_doc ="\
	min_ellipse.has_on_bounded_side( self , Point_2 p) -> bool\n\
	returns true, iff p lies properly inside min_ellipse.";

const char* Min_ellipse_has_on_unbounded_side_doc ="\
	min_ellipse.has_on_unbounded_side( self , Point_2 p) -> bool\n\
	returns true, iff p lies properly outside of min_ellipse.";

const char* Min_ellipse_has_on_boundary_doc ="\
	min_ellipse.has_on_boundary( self , Point_2 p) -> bool\n\
	returns true, iff p lies on the boundary of min_ellipse.";

const char* Min_ellipse_is_empty_doc ="\
	min_ellipse.is_empty( self ) -> bool\n\
	returns true, iff min_ellipse is empty( self ,this implies degeneracy).";

const char* Min_ellipse_is_degenerate_doc ="\
	min_ellipse.is_degenerate( self ) -> bool\n\
	returns true, iff min_ellipse is degenerate, i.e. \n\
	if min_ellipse is empty or equal to a segment equivalently \n\
	if the number of support points is less than 3.";

const char* Min_ellipse_clear_doc ="\
	deletes all points in min_ellipse and sets min_ellipse to the empty set.\n\
	Postcondition: min_ellipse.is_empty() = true.";

const char* Min_ellipse_is_valid_doc ="\
	min_ellipse.is_valid( self , bool verbose = false, int level = 0) -> bool\n\
	returns true, iff min_ellipse contains all points of its defining set P. \n\
	If verbose is true, some messages concerning the performed checks \n\
	are written to standard error stream. The second parameter level is\n\
	not used, we provide it only for consistency with interfaces of other classes.";

#endif // MIN_ELLIPSE_DOCS_H


