// Copyright( self ,c) 2005  INRIA Sophia-Antipolis( self ,France).
// All rights reserved.
//
// This file is part of cgal-python; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; version 2.1 of the License.
// See the file LICENSE.LGPL distributed with cgal-python.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// $Id: Py_Min_sphere_d_doc.h 125 2006-06-06 16:27:38Z nmeskini $
// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal-python/trunk/cgal-python/bindings/Geometric_Optimisation/Py_Min_sphere_d_doc.h $
//
// Author(s)     : Naceur Meskini
//========================================================================
#ifndef MIN_SPHERE_DOCS_H
#define MIN_SPHERE_DOCS_H
//=======================================================
// doc of Min_sphere_d class
//=======================================================

const char* Min_sphere_2_doc ="\
An object of the class Min_sphere_2 is the unique sphere of smallest\n\
volume enclosing a finite(multi)set of points in 2-dimensional Euclidean space   2. \n\
For a set P we denote by ms(P) the smallest sphere that contains all\n\
points of P. ms(P) can be degenerate, i.e. ms(P)=  if P=  and ms(P)={p} if P={p}.\n\
An inclusion-minimal subset S of P with ms(S)=ms(P) is called a support set, \n\
the points in S are the support points. A support set has size at most\n\
3, and all its points lie on the boundary of ms(P). \n\
In general, neither the support set nor its size are unique.\n\
for more details see a C++ doc:\n\
http://www.cgal.org/Manual/3.2/doc_html/cgal_manual/Optimisation_ref/Class_Min_sphere_d.html";

const char* Min_sphere_init_2_doc ="\
	min_sphere = Min_sphere_2() creates a variable of type Min_sphere_2 and initializes it to ms().";

const char* Min_sphere_number_of_points_doc ="\
	min_sphere.number_of_points( self) -> int\n\
	returns the number of points of min_sphere, i.e. |P|.";

const char* Min_sphere_number_of_support_points_doc ="\
	min_sphere.number_of_support_points( self ,) -> int\n\
	returns the number of support points of min_sphere, i.e. |S|.";

const char* Min_sphere_insert_doc ="\
min_sphere.insert( self , Point p) -> void\n\
	inserts p into min_sphere. \n\
	If p lies inside the current sphere, this is a constant-time operation,\n\
	otherwise it might take longer, but usually substantially less \n\
	than recomputing the smallest enclosing sphere from scratch.\n\
	Precondition: The dimension of p equals ambient_dimension() if\n\
 	min_sphere is not empty.\n\n\
	min_sphere.insert( self , list l) -> void\n\
	inserts a list of points l into min_sphere.";

const char* Min_sphere_ambient_dimension_doc ="\
	min_sphere.ambient_dimension( self ) -> int\n\
	returns the dimension of the points in P. \n\
	If min_sphere is empty, the ambient dimension is -1.";

const char* Min_sphere_center_doc ="\
	min_sphere.center( self ) -> Point\n\
	returns the center of min_sphere.\n\
	Precondition: min_sphere is not empty.";

const char* Min_sphere_squared_radius_doc ="\
	min_sphere.squared_radius( self ) -> double\n\
	returns the squared radius of min_sphere.\n\
	Precondition: min_sphere is not empty.";

const char* Min_sphere_bounded_side_doc ="\
	min_sphere.bounded_side( self , Point p) -> Bounded_side\n\
	returns CGAL.Kernel.ON_BOUNDED_SIDE, CGAL.Kernel.ON_BOUNDARY, \n\
	or CGAL.Kernel.ON_UNBOUNDED_SIDE iff p lies properly inside, on \n\
	the boundary, or properly outside of min_sphere, resp.\n\
	Precondition: if min_sphere is not empty, the dimension of p \n\
	equals ambient_dimension().";

const char* Min_sphere_has_on_bounded_side_doc ="\
	min_sphere.has_on_bounded_side( self , Point p) -> bool\n\
	returns true, iff p lies properly inside min_sphere.\n\
	Precondition: if min_sphere is not empty, the dimension of p \n\
	equals ambient_dimension().";

const char* Min_sphere_has_on_unbounded_side_doc ="\
	min_sphere.has_on_unbounded_side( self , Point p) -> bool\n\
	returns true, iff p lies properly outside of min_sphere.\n\
	Precondition: if min_sphere is not empty, the dimension of p \n\
	equals ambient_dimension().";

const char* Min_sphere_has_on_boundary_doc ="\
	min_sphere.has_on_boundary( self , Point p) -> bool\n\
	returns true, iff p lies on the boundary of min_sphere.\n\
	Precondition: if min_sphere is not empty, the dimension of p \n\
	equals ambient_dimension().";

const char* Min_sphere_is_empty_doc ="\
	min_sphere.is_empty( self ) -> bool\n\
	returns true, iff min_sphere is empty( self ,this implies degeneracy).";

const char* Min_sphere_is_degenerate_doc ="\
	min_sphere.is_degenerate( self ) -> bool\n\
	returns true, iff min_sphere is degenerate, i.e. \n\
	if min_sphere is empty or equal to a single point, equivalently \n\
	if the number of support points is less than 2.";

const char* Min_sphere_clear_doc ="\
	min_sphere.clear( self ) -> void\n\
	resets min_sphere to ms().";

const char* Min_sphere_is_valid_doc ="\
	min_sphere.is_valid( self , bool verbose = false, int level = 0) -> bool\n\
	returns true, iff min_sphere is valid. If verbose is true, some\n\
	messages concerning the performed checks are written to standard\n\
	error stream. The second parameter level is not used, we provide \n\
	it only for consistency with interfaces of other classes.";

const char* Min_sphere_3_doc ="\
An object of the class Min_sphere_3 is the unique sphere of smallest\n\
volume enclosing a finite(multi)set of points in 3-dimensional Euclidean space   3. \n\
For a set P we denote by ms(P) the smallest sphere that contains all\n\
points of P. ms(P) can be degenerate, i.e. ms(P)= if P= and ms(P)={p} if P={p}.\n\
An inclusion-minimal subset S of P with ms(S)=ms(P) is called a support set, \n\
the points in S are the support points. A support set has size at most\n\
4, and all its points lie on the boundary of ms(P). \n\
In general, neither the support set nor its size are unique.\n\
for more details see a C++ doc:\n\
http://www.cgal.org/Manual/3.2/doc_html/cgal_manual/Optimisation_ref/Class_Min_sphere_d.html";

const char* Min_sphere_init_3_doc ="\
	min_sphere = Min_sphere_3() creates a variable of type Min_sphere_3 and initializes it to ms().";

#endif // MIN_SPHERE_DOCS_H


